/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


function ImagesPage() // extend WizardPage
{
    var depends = "selected";
    WizardPage.call(this, depends);
}

ImagesPage.prototype =
{
    onPageShow: function()
    {
        if (this.dirty.page)
        {
            this.selected = new Array(model.selected.length);
            
            // page loaders
            var self = this;
            var headers = {"Referer": model.referer};
            var timeout = nsPreferences.getIntPref("extensions.thumbsdown.scriptwizard.requestTimeout");
            
            for each (var thumbnail in model.selected)
            {
                var target = thumbnail.target;
                if (! target.loader)
                {
                    var url = thumbnail.anchor.url;
                    let _thumbnail = thumbnail;
                    var onLoad = function(loader) { self.onPageLoad(loader, _thumbnail); }
                    var onError = function(loader) { self.onPageError(loader, _thumbnail); }
                    target.loader = new ResourceLoader("GET", url, headers, onLoad, onError, timeout);
                    target.loader.load();
                }
            }
            
            // clear left hand side richlistbox
            var left_rlbox = $("rlbox_thumbnails");
            while (left_rlbox.itemCount > 0)
                left_rlbox.removeItemAt(0);
            
            var deck = $("deck_pages");
            deck.selectedIndex = 0;
            
            // clear right hand side deck
            var rlbox = deck.firstChild;
            while (rlbox.nextSibling)
                deck.removeChild(rlbox.nextSibling);
            
            for (var i = 0; i < model.selected.length; i++)
            {
                var thumbnail = model.selected[i];
                var target = thumbnail.target;
                
                // left hand side
                var attrs = {maxwidth: 128, maxheight: 128, src: thumbnail.image.link, class: "show-throbber show-error"};
                var scaledimage = createXulElement("scaledimage", attrs);
                var rlitem = createXulElement("richlistitem", null, [scaledimage]);
                left_rlbox.appendChild(rlitem);
                
                // right hand side
                var rlitem = createXulElement("richlistitem", {class: "page"});
                var rlbox = createXulElement("richlistbox", {value: thumbnail.index}, [rlitem]);
                if (target.images)
                    this._populateImageListbox(rlbox, target.images);
                
                rlbox.suppressOnSelect = true;
                deck.appendChild(rlbox);
                if (target.image)
                    rlbox.selectedIndex = this.selected[i] = target.image.index;
                else
                    rlbox.selectedIndex = -1;
                rlbox.suppressOnSelect = false;
            }
            
            this._updateSelectedCount();
        }
    },
    
    onThumbnailSelect: function()
    {
        this._updateThumbnail();
        this._updateImage();
    },
    
    onPageLoad: function(loader, thumbnail)
    {
        try { this._onPageLoad(loader.content, thumbnail); }
        finally { loader.destroy(); }
    },
    
    onPageError: function(loader, thumbnail)
    {
        try { this._onPageLoad(null, thumbnail); }
        finally { loader.destroy(); }
    },
    
    _onPageLoad: function(doc, thumbnail)
    {
        var target = thumbnail.target;
        
        if (doc)
        {
            var images = [];
            
            for (var i = 0; i < doc.images.length; i++)
            {
                try
                {
                    var img = doc.images[i];
                    image =
                    {
                        id: img.id,
                        link: img.src,
                        url: ioSvc.newURI(img.src, null, null).QueryInterface(Ci.nsIURL),
                        index: images.length
                    }
                    images.push(image);
                }
                catch (e) { /* ignore */ }
            }
            
            target.images = images;
        }
        
        if (page != pages.images)
        {
            this.dirty.page = true;
            return;
        }
        
        // find thumbnail's rlbox
        var deck = $("deck_pages");
        var rlbox = deck.firstChild.nextSibling;
        while (rlbox && (rlbox.getAttribute("value") != thumbnail.index))
            rlbox = rlbox.nextSibling;
        
        this._populateImageListbox(rlbox, target.images);
    },
    
    onImageSelect: function(e)
    {
        var rlbox = e.target;
        var thumbnailIndex = rlbox.parentNode.selectedIndex - 1;
        var rlitem = rlbox.selectedItem;
        var imageIndex = parseInt(rlitem.value);
        this.selected[thumbnailIndex] = imageIndex;
        this._updateImage();
        this._updateSelectedCount();
        wizard.updateWizardButtons();
    },
    
    canAdvance: function()
    {
        return (this._count(this.selected) >= 2);
    },
    
    _populateImageListbox: function(rlbox, images)
    {
        if (images && (images.length))
        {
            var self = this;
            
            // remove dummy item
            rlbox.removeChild(rlbox.firstChild);
            
            for (var i = 0; i < images.length; i++)
            {
                var attrs = {maxwidth: 128, maxheight: 128, src: images[i].link, class: "show-throbber show-error"};
                var scaledimage = createXulElement("scaledimage", attrs);
                var rlitem = createXulElement("richlistitem", {value: i}, [scaledimage]);
                
                rlbox.appendChild(rlitem);
            }
            
            rlbox.addEventListener("select", function(e) { self.onImageSelect(e); }, false);
        }
        else
        {
            var msg = (images ? _("error.pageHasNoImages") : _("error.pageLoadError"));
            var label = createXulElement("label", {value: msg});
            rlbox.firstChild.appendChild(label);
            rlbox.disabled = true;
        }
    },
    
    _updateThumbnail: function()
    {
        var rlbox = $("rlbox_thumbnails");
        var index = rlbox.currentIndex;
        $("deck_pages").selectedIndex = index + 1;
        var thumbnail = model.selected[index];
        $("lbl_images_thumbnail").value = thumbnail.image.link;
        $("lbl_images_link").value = thumbnail.anchor.link;
    },
    
    _updateImage: function()
    {
        var rlbox = $("deck_pages").selectedPanel;
        var rlitem = rlbox.selectedItem;
        if (rlitem)
        {
            var scaledimage = rlitem.firstChild;
            $("lbl_images_image").value = scaledimage.src;
            $("lbl_images_imageSize").value = (scaledimage.state == "loaded" ? _("imageSize", scaledimage.naturalWidth, scaledimage.naturalHeight) : "");
        }
        else
        {
            $("lbl_images_image").value = "";
            $("lbl_images_imageSize").value = "";
        }
    },
    
    _updateSelectedCount: function()
    {
        $("lbl_images_selectedCount").value = _("count", this._count(this.selected), this.selected.length);
    },
    
    _count: function(arr)
    {
        return count(arr, function(a) { return (a !== undefined); });
    },
    
    onPageAdvanced: function()
    {
        var selectedImages = [];
        
        for (var i = 0; i < this.selected.length; i++)
        {
            var thumbnail = model.selected[i];
            var imageIndex = this.selected[i];
            if (typeof(imageIndex) == "number")
            {
                thumbnail.target.image = thumbnail.target.images[imageIndex];
                selectedImages.push(thumbnail);
            }
        }
        
        model.images = selectedImages;
        return true;
    }
}

extend(ImagesPage, WizardPage);

pages.images = new ImagesPage();
