/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


function NamePage() // extend WizardPage
{
    var depends = "selected linkUrlTokens script";
    WizardPage.call(this, depends);
}

NamePage.prototype =
{
    get name() { return $("tbox_name") },
    
    onPageShow: function()
    {
        if (this.firstRun)
            $("lbl_name_warning").style.visibility = "hidden";
        
        if (this.dirty.selected)
        {
            var rlbox = $("rlbox_name");
            while (rlbox.itemCount > 0)
                rlbox.removeItemAt(0);
            for each (var thumbnail in model.selected)
            {
                var rlitem = rlbox.appendItem(thumbnail.anchor.link);
                rlitem.firstChild.crop = "none";
            }
        }
        
        if (this.dirty.linkUrlTokens || this.dirty.script)
        {
            var name = ((model.mode == MODE_MODIFY) ? model.script.name : scriptWizard.getName(model.linkUrlTokens));
            this.name.defaultValue = name;
            $("tbox_name").reset();
        }
        
        if (this.dirty.page)
        {
            this.checkName();
            this._updateButtons();
        }
    },
    
    onNameInput: function()
    {
        if (this.updateTimer)
            this.updateTimer.cancel();
        
        this.updateTimer = callLater(INPUT_DELAY, this, this.checkName);
        
        this._updateButtons();
        wizard.updateWizardButtons();
    },
    
    resetName: function()
    {
        this.name.reset();
        this._updateButtons();
        wizard.updateWizardButtons();
    },
    
    checkName: function()
    {
        delete this.updateTimer;
        
        if (page != pages.name)
            return;
        
        var duplicate = false;
        if (this.name.value.length > 0)
        {
            // TODO: don't use sqlExec directly here
            var scripts = scriptMgr.sqlExec.fetchAll("getScriptsWithName", this.name.value);
            if (scripts.length == 1)
                duplicate = ((model.mode == MODE_MODIFY) ? (scripts[0].id != model.script.id) : true);
            else if (scripts.length > 1)
                duplicate = true;
        }
        
        $("lbl_name_warning").style.visibility = (duplicate ? "visible" : "hidden");
    },
    
    canAdvance: function()
    {
        return (this.name.value.length > 0);
    },
    
    _updateButtons: function()
    {
        $("btn_name_reset").disabled = (this.name.value == this.name.defaultValue);
    },
    
    onPageAdvanced: function()
    {
        model.name = this.name.value;
        return true;
    }
}

extend(NamePage, WizardPage);

pages.name = new NamePage();
